package com.miaxis.app.controller.question;

import com.miaxis.common.constant.Constants;
import com.miaxis.common.core.controller.BaseController;
import com.miaxis.common.core.domain.Response;
import com.miaxis.common.core.page.ResponsePageInfo;
import com.miaxis.question.domain.QuestionInfo;
import com.miaxis.question.dto.QuestionInfoDTO;
import com.miaxis.question.dto.QuestionInfoFlDTO;
import com.miaxis.question.dto.QuestionInfoMiDTO;
import com.miaxis.question.dto.QuestionInfoTestDTO;
import com.miaxis.question.service.IQuestionInfoService;
import com.miaxis.question.vo.QuestionInfoDfVO;
import com.miaxis.question.vo.QuestionInfoFlVO;
import com.miaxis.question.vo.QuestionInfoJxVO;
import com.miaxis.question.vo.QuestionInfoSxVO;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 【题库】Controller
 *
 * @author miaxis
 * @date 2021-10-20
 */
@RestController
@RequestMapping(Constants.STUDENT_PREFIX+"/question/info")
@Api(tags={"【H5-题库】"})
public class QuestionInfoController extends BaseController {
    @Autowired
    private IQuestionInfoService questionInfoService;

    /**
     * 查询题库列表
     */
    @GetMapping("/list")
    @ApiOperation("查询题库列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum",value = "当前页码" ,dataType = "int", paramType = "query", required = false),
            @ApiImplicitParam(name = "pageSize",value = "每页数据量" , dataType = "int", paramType = "query", required = false),
    })
    public ResponsePageInfo<QuestionInfo> list(@ModelAttribute QuestionInfoDTO questionInfoDTO){
        startPage();
        List<QuestionInfo> list = questionInfoService.selectQuestionInfoList(questionInfoDTO);
        return toResponsePageInfo(list);
    }

//    /**
//     * 导出题库列表
//     */
//    @Log(title = "题库", businessType = BusinessTypeEnum.EXPORT)
//    @GetMapping("/export")
//    @ApiOperation("导出题库列表Excel")
//    public Response<String> export(@ModelAttribute QuestionInfo questionInfo){
//        List<QuestionInfo> list = questionInfoService.selectQuestionInfoList(questionInfo);
//        ExcelUtil<QuestionInfo> util = new ExcelUtil<QuestionInfo>(QuestionInfo.class);
//        return util.exportExcel(list, "info");
//    }

    /**
     * 获取题库详细信息
     */
    @GetMapping(value = "/{id}")
    @ApiOperation("获取题库详细信息")
    public Response<QuestionInfo> getInfo(
            @ApiParam(name = "id", value = "题库参数", required = true)
            @PathVariable("id") Long id
    ){
        return Response.success(questionInfoService.getById(id));
    }

    /**
     * 获取题库详细信息多个题库
     */
    @GetMapping(value = "/getQuestionInfoByIds")
    @ApiOperation("获取题库详细信息批量取题")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids",value = "题目ids" ,dataType = "List", required = true)})
    public ResponsePageInfo<QuestionInfo> getQuestionInfoByIds(Long[] ids){
        List<QuestionInfo> list = questionInfoService.selectQuestionInfoListByIds(ids);
        return toResponsePageInfo(list);
    }


    /**
     * 新增题库
     */
//    @PreAuthorize("@ss.hasPermi('qustion:info:add')")
//    @Log(title = "题库", businessType = BusinessTypeEnum.INSERT)
//    @PostMapping
//    @ApiOperation("新增题库")
//    public Response<Integer> add(@RequestBody QuestionInfo questionInfo){
//        return toResponse(questionInfoService.save(questionInfo) ? 1 : 0);
//    }

    /**
     * 修改题库
     */
//    @PreAuthorize("@ss.hasPermi('qustion:info:edit')")
//    @Log(title = "题库", businessType = BusinessTypeEnum.UPDATE)
//    @PutMapping
//    @ApiOperation("修改题库")
//    public Response<Integer> edit(@RequestBody QuestionInfo questionInfo){
//        return toResponse(questionInfoService.updateById(questionInfo) ? 1 : 0);
//    }

    /**
     * 删除题库
     */
//    @PreAuthorize("@ss.hasPermi('qustion:info:remove')")
//    @Log(title = "题库", businessType = BusinessTypeEnum.DELETE)
//	@DeleteMapping("/{ids}")
//    @ApiOperation("删除题库")
//    public  Response<Integer> remove(
//            @ApiParam(name = "ids", value = "题库ids参数", required = true)
//            @PathVariable Long[] ids
//    ){
//        return toResponse(questionInfoService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
//    }


    /**
     * 获取精选分类
     */
    @GetMapping(value = "/selectJxQuestionInfo")
    @ApiOperation("获取精选题分类")
    public Response<List<QuestionInfoJxVO>> selectJxQuestionInfo(@ModelAttribute QuestionInfoFlDTO questionInfoFlDTO) {
        return Response.success(questionInfoService.selectJxQuestionInfo(questionInfoFlDTO));
    }


    /**
     * 获取地方分类
     */
    @GetMapping(value = "/selectDfQuestionInfo")
    @ApiOperation("获取地方题分类")
    public Response<List<QuestionInfoDfVO>> selectDfQuestionInfo(@ModelAttribute QuestionInfoFlDTO questionInfoFlDTO) {
        return Response.success(questionInfoService.selectDfQuestionInfo(questionInfoFlDTO));
    }

    /**
     * 获取分类专题分类
     */
    @GetMapping(value = "/selectFlQuestionInfo")
    @ApiOperation("获取分类专题分类")
    public Response<List<QuestionInfoFlVO>> selectFlQuestionInfo(@ModelAttribute QuestionInfoFlDTO questionInfoFlDTO) {
        return Response.success(questionInfoService.selectFlQuestionInfo(questionInfoFlDTO));
    }

    /**
     * 获取顺序练习分类
     */
    @GetMapping(value = "/selectSxQuestionInfo")
    @ApiOperation("获取顺序练习分类")
    public Response<List<QuestionInfoSxVO>> selectSxQuestionInfo(@ModelAttribute QuestionInfoFlDTO questionInfoFlDTO) {


        return Response.success(questionInfoService.selectSxQuestionInfo(questionInfoFlDTO));
    }


    /**
     * 查询题库列表（非会员）
     */
    @GetMapping("/normalList")
    @ApiOperation("查询题库列表（非会员）")
    public ResponsePageInfo<QuestionInfo> normalList(@ModelAttribute QuestionInfoDTO questionInfoDTO){
        List<QuestionInfo> list = questionInfoService.selectNormalQuestionInfoList(questionInfoDTO);
        return toResponsePageInfo(list);
    }


    /**
     * 查询模拟考试题库
     */
    @GetMapping("/selectTestQuestionInfo")
    @ApiOperation("查询模拟考试题库")
    public ResponsePageInfo<QuestionInfo> selectTestQuestionInfo(@ModelAttribute QuestionInfoTestDTO questionInfoTestDTO){
        List<QuestionInfo> list = questionInfoService.selectTestQuestionInfo(questionInfoTestDTO);
        return toResponsePageInfo(list);
    }


    /**
     * 查询免费题库
     */
    @GetMapping("/selectFreeQuestionInfo")
    @ApiOperation("查询免费题库")
    public ResponsePageInfo<QuestionInfo> selectFreeQuestionInfo(){
        List<QuestionInfo> list = questionInfoService.selectFreeQuestionInfo();
        return toResponsePageInfo(list);
    }


    /**
     * 查询秘密卷
     */
    @GetMapping("/selectMiQuestionInfoList")
    @ApiOperation("查询秘密卷")
    public ResponsePageInfo<QuestionInfo> selectMiQuestionInfoList(@ModelAttribute QuestionInfoMiDTO questionInfoMiDTO){
        List<QuestionInfo> list = questionInfoService.selectMiQuestionInfoList(questionInfoMiDTO);
        return toResponsePageInfo(list);
    }


}

