package com.miaxis.wx.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.img.ImgUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.extra.qrcode.QrCodeUtil;
import cn.hutool.extra.qrcode.QrConfig;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.miaxis.common.utils.http.HttpUtils;
import com.miaxis.feign.dto.WxGzhMini;
import com.miaxis.feign.dto.WxMessageCusom;
import com.miaxis.feign.dto.WxMessageTemplate;
import com.miaxis.feign.dto.WxQrTicket;
import com.miaxis.feign.service.IWxSendService;
import com.miaxis.wx.domain.WxForeverCode;
import com.miaxis.wx.service.IWxMessageEvenService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.awt.*;
import java.io.File;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * @author wwl
 * @version 1.0
 * @date 2021/10/27 14:35
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class WxMessageEvenServiceImpl implements IWxMessageEvenService {

    private final static Integer expireTime = 2592000;//30天

    private final RedisTemplate redisTemplate;

    private final IWxSendService wxSendService;








    /**
     * 扫码关注事件
     * @param fromUserName 发送方帐号（一个OpenID）
     * @param eventKey
     * @param token 凭证
     * @param userName 微信名称
     * @return
     */
    @Override
    public String scanSubscribeEvent(String fromUserName, String eventKey,String token,String userName) {

        String message = "";
        //如果扫自己
        if (fromUserName.equals(eventKey)){
            //推送绑定模板信息到上级
            return "不能成为自己的下级！";
        }
        //判断是否存在父、子级推广关系
        String openid = eventKey;
        if (true) {
            //保存推广关系表


            //推送绑定模板信息到上级
            messageTemplateSend(openid, token, userName);
            message = "欢迎关注金牌车教，您的学车好伙伴！";
        }
        else {
            message =  "已存在上级绑定关系！";
        }
        return message;
    }


    /**
     * click获取推广二维码事件
     * @param fromUserName 发送方帐号（一个OpenID）
     * @param token 凭证
     * @param wxForeverCode
     * @return
     */
    @Override
    public String gainTicketEvent(String fromUserName, String token, boolean isForever, WxForeverCode wxForeverCode) {

        //发送客服信息
        messageCustomSend(fromUserName,token);


        return null;
    }

    private WxQrTicket createForeverTicket(String fromUserName) {
        WxQrTicket wxQrTicket = new WxQrTicket();
        wxQrTicket.setAction_name("QR_LIMIT_STR_SCENE");//临时字符串参数
        JSONObject jsonObject1 = new JSONObject();
        JSONObject jsonObject2 = new JSONObject();
        jsonObject1.put("scene_str",fromUserName);
        jsonObject2.put("scene",jsonObject1);
        wxQrTicket.setAction_info(jsonObject2);
        return wxQrTicket;
    }

    private WxQrTicket createShortTimeTicket(String fromUserName) {
        WxQrTicket wxQrTicket = new WxQrTicket();
        wxQrTicket.setExpire_seconds(expireTime);
        wxQrTicket.setAction_name("QR_STR_SCENE");//临时字符串参数
        JSONObject jsonObject1 = new JSONObject();
        JSONObject jsonObject2 = new JSONObject();
        jsonObject1.put("scene_str",fromUserName);
        jsonObject2.put("scene",jsonObject1);
        wxQrTicket.setAction_info(jsonObject2);
        return wxQrTicket;
    }


    /**
     * 发送客服消息
     * @param fromUserName
     * @param token
     */
    private void messageCustomSend(String fromUserName,String token){

        WxMessageCusom wxMessageCusom = new WxMessageCusom();
        wxMessageCusom.setTouser(fromUserName);
        wxMessageCusom.setMsgtype("text");
        HashMap<String, Object> contentMap = new HashMap<>();
        contentMap.put("content","正在努力生成推广码请稍等...");
        wxMessageCusom.setText(contentMap);
        String send = wxSendService.messageCustomSend(token, wxMessageCusom);
        JSONObject jsonObject = JSONObject.parseObject(send);
        log.info("回复客服消息：[{}],响应信息：[{}]",contentMap,jsonObject);
    }

    /**
     * 发送模板信息
     * @param openid
     * @param token
     * @param nickName
     */
    private void messageTemplateSend(String openid,String token,String nickName){

        WxMessageTemplate messageTemplate = new WxMessageTemplate();
        messageTemplate.setTouser(openid);
        messageTemplate.setTemplate_id("P4nKRB6L3dmhBjEj1VeE0gr9yC11Z0uBTAuu-XKmIjs");
        messageTemplate.setUrl("https://jpcj-h5.zzxcx.net/myIntegral");
        WxGzhMini wxGzhMini = new WxGzhMini();
        messageTemplate.setMiniprogram(wxGzhMini);

        JSONObject jsonObject = new JSONObject();
        JSONObject first = new JSONObject();
        JSONObject keyword1 = new JSONObject();
        JSONObject keyword2 = new JSONObject();
        JSONObject keyword3 = new JSONObject();
        first.put("value","您好，已成功绑定您的账号！");
        first.put("color","#173177");
        keyword1.put("value",nickName);
        keyword1.put("color","#173177");
        keyword2.put("value", DateUtil.format(new Date(), "yyyy-MM-dd HH:mm"));
        keyword2.put("color","#173177");
        keyword3.put("value","有新用户已成为你的下级！");
        keyword3.put("color","#173177");
        jsonObject.put("first",first);
        jsonObject.put("keyword1",keyword1);
        jsonObject.put("keyword2",keyword2);
        jsonObject.put("keyword3",keyword3);
        messageTemplate.setData(jsonObject);
        String result = wxSendService.messageTemplateSend(token,messageTemplate);
        JSONObject parseObject = JSONObject.parseObject(result);
        log.info("模板消息：[{}],响应信息：[{}]",messageTemplate,parseObject);

    }




}
